Class: DuckTest::FrameWork::QueueEvent

Inherits:
Object
  • Object
show all
Defined in:
lib/duck_test/frame_work/queue_event.rb

Overview

A QueueEvent is triggered when directories / files have changed and contains a list of the files that have changed.

Instance Attribute Summary collapse

Instance Method Summary collapse

Constructor Details

#initialize(source, files) ⇒ QueueEvent

Initialize a new QueueEvent

Parameters:

  • source (Object)

    A reference to the calling object.

  • files (Array)

    A list of files that have changed and require action.



18
19
20
21
22
23
24
25
# File 'lib/duck_test/frame_work/queue_event.rb', line 18

def initialize(source, files)
  super()

  self.source = source
  self.files = files.uniq

  return self
end

Instance Attribute Details

#filesObject



11
12
13
# File 'lib/duck_test/frame_work/queue_event.rb', line 11

def files
  @files
end

#sourceObject



8
9
10
# File 'lib/duck_test/frame_work/queue_event.rb', line 8

def source
  @source
end